body, html {
  height: 100%;
  display: flex;
  flex-direction: column;
}

.main-content {
  flex-grow: 1;
}

.bottom-section {
  flex-grow: 0;
  margin: auto;
  padding-bottom: 32px;
  width: calc(100% - 3rem);
}

.navbar {
  background: #0abf53;
  min-height: 108px;
  background-image: url("leaf-bg.svg");
  background-repeat: no-repeat;
}

.navbar-item {
  margin-top: 62px;
  color: white;
}

button[type="submit"] {
  width: 100%;
  height: 44px;
  border-radius: 2px;
  background-color: #0abf53;
  color: white;
}

input[type=text], input[type=number] {
  height: 44px;
  width: 100%;
  border-radius: 2px;
}

.control .icon {
  margin-top: 4px;
}

.receipt label {
  font-variant: small-caps;
  font-weight: bold;
}

.receipt .field-value {
  text-transform: capitalize;
}

.flex {
  display: flex;
  flex-direction: column;
}

.flex-spacer {
  flex-grow: 1;
}

.button-row {
  display: flex;
  flex-direction: row;
  gap: 8px 8px;
}

.button-row button {
  display: block;
  height: 44px;
  font-size: 14px;
  color: white;
  border: 0;
  border-radius: 2px;;
}

.button-row .spacer {
  width: 8px;
}

.button-row .button-main {
  flex-grow: 1;
  background-color: #490be3;
}

.button-row .button-secondary {
  flex-grow: 0;
  min-width: 44px;
  background-color: #0abf53;
}

.refresh-icon {
  background: url("icon-refresh.svg") center center no-repeat;
}