{% load static %}
<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1" />

    {% block 'title' %}
    <title>SEP24 reference server</title>
    {% endblock %}

    <link
      rel="stylesheet"
      href="https://cdnjs.cloudflare.com/ajax/libs/bulma/0.7.5/css/bulma.min.css"
    />
    <link rel="stylesheet" href="{% static "base.css" %}" />
    <script
      defer
      src="https://use.fontawesome.com/releases/v5.3.1/js/all.js"
    ></script>
  </head>
  <body>
    {% block "header" %}
    <nav class="navbar" role="navigation" aria-label="main navigation">
      <div class="navbar-brand">
        <a class="navbar-item" href="#">
          Loanly Bay Leaf Bank Corp
        </a>
      </div>
    </nav>
    {% endblock %}
    <div class="main-content">
      {% block "content" %}
      {% endblock %}
    </div>
    <div class="bottom-section">
      {% block "footer" %}
      {% endblock %}
    </div>
  </body>
</html>
