{% extends "base.html" %}

{% block "title" %}
<title>Interactive Deposit Info Form</title>
{% endblock %}


{% block "content" %}
<section class="section">
  <p>
    This is the interactive deposit flow. Please enter the amount you would like to deposit.
  </p>
  <br />
  <form method="post" action="">
    {% csrf_token %}

    {% for field in form %}
    <div class="field">
      <label class="label">{{ field.label }}</label>
      <div class="control has-icons-left">
        {{ field }}
        <span class="icon is-left">
          <i class="fas fa-dollar-sign"></i>
        </span>
      </div>

      {% if field.errors %}
      <p class="help is-danger">
        {% for error in field.errors %}
        {{ error }}
        {% endfor %}
      </p>
      {% endif %}

    </div>
    {% endfor %}

    <div class="field at-bottom">
      <div class="control">
        <button class="button" type="submit">Submit</button>
      </div>
    </div>
  </form>
</section>
{% endblock %}